
# -----------------------------------------------------------------------------
# Copyright (c) 2009 by Christine Walter, Matthias Klusch
# Saarland University and German Research Center for Artificial Intelligence,
# Saarbruecken, Germany
#
# All rights reserved. No warranties will be given on any issues arising out 
# of the use of this software product. Permission to use, copy, modify and 
# distribute this software product for non-commercial purposes is hereby 
# granted, provided that the above copyright notice appears in all copies and 
# respective publications.
# All commercial trades with this product without specific written prior 
# permission are prohibited.
# -----------------------------------------------------------------------------



# ===== coala_update_history_data
# ===== update some data after one negotiation, for example 
# ===== trust, total_utility, etc
proc coala_update_history_data {} {
	global all_in_nego payoff_vector coal_struct is_opp act_mal
	global total_utility total_utility_change num_building_coalition_nego algorithm trial_time
	global reputation reputation_info agents_list
	global current_utility_change uc_vals algorithm
	global this_neg_ro this_neg_gs met case_uc_agent case_uc_gs case_uc_ro case_uc total_cases
	global simulation_name neg_no pred_uc

	foreach coal $coal_struct {
		foreach agent $coal {
			set i [lsearch $all_in_nego $agent]
			set pay [lindex $payoff_vector $i]
			set total_utility($agent) [expr $total_utility($agent) + $pay]
		}
	}

	if {$algorithm == "BSCA-D"} {
		return
	}
	# ===== update values for trust rating & reputation rating
	foreach coal $coal_struct {
		foreach agent $coal {
			set value($agent) 0.0
		}
		if {[llength $coal] <= 1} {
			continue
		}
		global infos_in_nego 
		foreach agent $coal {
			foreach info_unit $infos_in_nego($agent) {
				set price [lindex $info_unit 0]
				set access [lindex $info_unit 1]
				set search [lindex $info_unit 2]

				foreach o_agent $coal {
					if {$o_agent == $agent} {
						continue
					}
					if {[lsearch $search $o_agent] == -1} {
						continue
					}
					# ==== if the info accessible, then add price to reduced_sum
					if [info_is_accessible2 $o_agent $coal $access] {
						set value($agent) [expr $value($agent) + ($price + 0.0) / (2.0 * [llength $coal])]
						set value($o_agent) [expr $value($o_agent) + ($price + 0.0) / (2.0 * [llength $coal])]
					}
				}
			}
		}
		foreach agent $coal {
			foreach o_agent $coal {
				if {$o_agent == $agent} {
					continue
				}
				set index [list $agent $o_agent]
				set total_utility_change($index) [expr $total_utility_change($index) + $value($o_agent)]
				# -- for RC, now have final current_utility_change
				if {$algorithm == "BSCA-RC"} {
					set current_utility_change($index) [expr $current_utility_change($index) + $value($o_agent)]
					lappend met($agent) $o_agent
				}
				# ==== write reputation info
				if {$algorithm == "BSCA-TR" || $algorithm == "BSCA-TR*" || $algorithm == "BSCA-Opp" || $algorithm == "BSCA-RC"} {
					foreach a_agent $agents_list {
						if {$o_agent == $a_agent || $agent == $a_agent} {	
							continue
						}
						set index2 [list $o_agent $a_agent]
						set index3 [list $agent $o_agent $a_agent]
						if {$trial_time($o_agent) >= $num_building_coalition_nego($index2)} {
							set reputation_info($index3) 0
						} else {
							set reputation_info($index3) [expr $total_utility_change($index2) / $num_building_coalition_nego($index2)]
						}
					}
				}
			}
		}
	}

	# ==== calculate reputation 
	if {$algorithm == "BSCA-TR" || $algorithm == "BSCA-TR*" || $algorithm == "BSCA-Opp" || $algorithm == "BSCA-RC"} {
		foreach agent $agents_list {
			set rep_sender {}
			foreach agent1 $agents_list {
				if {$agent == $agent1} {
					continue
				}
				set index [list $agent $agent1]
				if {$trial_time($agent) >= $num_building_coalition_nego($index)} {     
					continue
				}
				# ===== accept only reputation information of trustworthy agents who are not in probation period
				if {[expr $total_utility_change($index) / $num_building_coalition_nego($index)] < 0} {
					continue
				}
				lappend rep_sender $agent1
			}
			if {[llength $rep_sender] == 0} {
				continue
			}
			foreach agent1 $agents_list {
				if {$agent == $agent1} {
					continue
				}
				set rep 0 
				set num_sender 0
				foreach agent2 $rep_sender {
					if {$agent == $agent2 || $agent1 == $agent2} {
						continue
					}
					set rep [expr $rep + $reputation_info([list $agent $agent2 $agent1])]
					incr num_sender
				}
				if {$num_sender > 0} {
					set reputation([list $agent $agent1]) [expr $rep / ($num_sender + 0.0)]
				}
			}
		}
	}
	# -- opportunists have learned something new
	foreach agent $agents_list {
		if {$is_opp($agent)} {
			set i [lsearch $all_in_nego $agent]
			set pay [lindex $payoff_vector $i]
			coala_opp_add_experience $agent $pay
		}
	}

	global predict_tr predict_nbc predict_other_gszero predict_other_nodata predict_other_probation

	# -- round uc, update counters, write new cases to logfile, unset vars for next negotiation
	if {$algorithm == "BSCA-RC"} {
		foreach agent $agents_list {
			# -- malicious agents consider everybody reliable
			if {$act_mal($agent) && !$is_opp($agent)} {
				continue
			}
			# -- add new data to file; if no file exists, create it
			set uc_logfile [open "logs/$simulation_name\.utilitychange$agent\.log" a]
			coala_puts $uc_logfile "begin_negotiation $neg_no"
			foreach agent2 $agents_list {
				if {[lsearch $met($agent) $agent2] != -1 && $agent != $agent2} {
				# -- if there is data for the agent, use it
					set index [list $agent $agent2]
					if {![info exists this_neg_gs($index)] || ![info exists this_neg_ro($index)]} {
						continue
					}
					set uc [coala_round $current_utility_change($index)]
					if {[lsearch $uc_vals($agent) $uc] == -1} {
						lappend uc_vals($agent) $uc
					}
					set gs $this_neg_gs($index)
					set ro $this_neg_ro($index)
					set index1 [list $agent $agent2 $uc]
					set index2 [list $agent $gs $uc]
					set index3 [list $agent $ro $uc]
					set index4 [list $agent $uc]
					if {![info exists case_uc_agent($index1)]} {
						set case_uc_agent($index1) 0.0
					}
					if {![info exists case_uc_gs($index2)]} {
						set case_uc_gs($index2) 0.0
					}
					if {![info exists case_uc_ro($index3)]} {
						set case_uc_ro($index3) 0.0
					}
					if {![info exists case_uc($index4)]} {
						set case_uc($index4) 0.0
					}
					if {![info exists total_cases($agent)]} {
						set total_cases($agent) 0.0
					}
					set case_uc_agent($index1) [expr $case_uc_agent($index1) + 1.0]
					set case_uc_gs($index2) [expr $case_uc_gs($index2) + 1.0]
					set case_uc_ro($index3) [expr $case_uc_ro($index3) + 1.0]
					set case_uc($index4) [expr $case_uc($index4) + 1.0]
					set total_cases($agent) [expr $total_cases($agent) + 1.0]
					coala_puts $uc_logfile [list $agent2 $gs $ro $uc]
					# -- update counters
					set pred_val [lindex $pred_uc($index) 0]
					set pred_case [lindex $pred_uc($index) 1]
					set diff_pred_res [expr $uc - $pred_val]
					if {$pred_case == "probation"} {
						incr predict_other_probation($agent)
					}
					if {$pred_case == "nodata"} {
						incr predict_other_nodata($agent)
					}
					if {$pred_case == "gszero"} {
						incr predict_other_gszero($agent)
					}
					if {$pred_case == "tr"} {
						set right_guesses [lindex $predict_tr($agent) 0]
						set near_misses [lindex $predict_tr($agent) 1]
						set wrong_guesses [lindex $predict_tr($agent) 2]
						if {$diff_pred_res == 0} {
							incr right_guesses
						} else {
							if {$diff_pred_res == 1 || $diff_pred_res == -1} {
								incr near_misses
							} else {
								incr wrong_guesses
							}
						}
						set predict_tr($agent) [list $right_guesses $near_misses $wrong_guesses]
					}
					if {$pred_case == "nbc"} {
						set right_guesses [lindex $predict_nbc($agent) 0]
						set near_misses [lindex $predict_nbc($agent) 1]
						set wrong_guesses [lindex $predict_nbc($agent) 2]
						if {$diff_pred_res == 0} {
							incr right_guesses
						} else {
							if {$diff_pred_res == 1 || $diff_pred_res == -1} {
								incr near_misses
							} else {
								incr wrong_guesses
							}
						}
						set predict_nbc($agent) [list $right_guesses $near_misses $wrong_guesses]
					}
				}
			}
		coala_puts $uc_logfile "probation_cases $predict_other_probation($agent)"
		coala_puts $uc_logfile "nodata_cases $predict_other_nodata($agent)"
		coala_puts $uc_logfile "gszero_cases $predict_other_gszero($agent)"
		coala_puts $uc_logfile "tr_predictions $predict_tr($agent)"
		coala_puts $uc_logfile "nbc_predictions $predict_nbc($agent)"
		coala_puts $uc_logfile "end_negotiation $neg_no"
		close $uc_logfile
		}
		coala_unset this_neg_ro
		coala_unset this_neg_gs
	}
}

# ===== if 'agents' leave the coalition 'coal', this function
# ===== corrects the informations about founders of 'coal' and
# ===== all subcoalitions of 'coal'
proc coala_correct_founder {coal agents} {
	global founder_of

	if {[llength $coal] <= 1 || [llength $agents] == 0} {
		return
	} 
	set new_coal $coal
	set C1 [lindex $founder_of($coal) 0]
	set new_C1 $C1
	set C2 [lindex $founder_of($coal) 1]
	set new_C2 $C2
	set leave_C1 {}
	set leave_C2 {}
	foreach agent $agents {
		set i [lsearch $new_C1 $agent]
		set j [lsearch $new_coal $agent]
		if {$i != -1} {
			lappend leave_C1 $agent 
			set new_C1 [lreplace $new_C1 $i $i]
			set new_coal [lreplace $new_coal $j $j]
		} else {
			set i [lsearch $new_C2 $agent]
			lappend leave_C2 $agent 
			set new_C2 [lreplace $new_C2 $i $i]
			set new_coal [lreplace $new_coal $j $j]
		}
	}
	
	if {[llength $new_C1] > 0 && [llength $new_C2] > 0} {
		set founder_of($new_coal) [list $new_C1 $new_C2] 
	}
	if {[llength $new_coal] <= 1} {
		return
	}
	coala_correct_founder $C1 $leave_C1 
	coala_correct_founder $C2 $leave_C2 
}

proc coala_correct_coal_values_after_agent_leaving {coal agent} {
	global logfile v_c founder_of

	if {[llength $coal] == 1} {
		# coalition value for leaving agent = 0
		set v_c($coal) 0
		if {$is_opp($coal)} {
			set index1 [concat $coal $coal]
			set real_v_c($index1) 0
		}
		set index1 [concat $coal $coal]
		return
	} 

	set i [lsearch $coal $agent]
	set new_coal [lreplace $coal $i $i]
	set v_c($new_coal) [coala_calc_coal_value $new_coal]
	set C1 [lindex $founder_of($coal) 0]
	set C2 [lindex $founder_of($coal) 1]
	set i [lsearch $C1 $agent]
	if {$i != -1} {
		coala_correct_coal_values_after_agent_leaving $C1 $agent
	} else {
		coala_correct_coal_values_after_agent_leaving $C2 $agent
	}
}

proc coala_correct_coal_values {coal pagent} {
	global founder_of v_c

	set v_c($coal) [coala_calc_coal_value $coal]
	if {[llength $coal] == 1} {
		return
	} 

	set C1 [lindex $founder_of($coal) 0]
	set C2 [lindex $founder_of($coal) 1]
	if {[lsearch $C1 $pagent] > -1} {
		coala_correct_coal_values $C1 $pagent
	} else {		
		coala_correct_coal_values $C2 $pagent
	}
}		 

proc coala_get_rational_partition {coal partition} {
	global founder_of v_c bsv
	upvar $partition ptt

	if {[llength $coal] == 0} {
		return
	} 	
	# calculate bsv
	set rational 1
	coala_calc_rbsv $coal bsv 0 founder_of
	# is coal rational?
	foreach agent $coal {
		if {$v_c($agent) > $bsv($agent)} {
			set rational 0
		}
	}
	if {$rational == 1} {
		lappend ptt $coal
		return
	}		
	set C1 [lindex $founder_of($coal) 0]
	set v_c($C1) [coala_calc_coal_value $C1]
	set C2 [lindex $founder_of($coal) 1]
	set v_c($C2) [coala_calc_coal_value $C2]
	coala_get_rational_partition $C1 ptt
	coala_get_rational_partition $C2 ptt
}		 

proc coala_get_coals {coalnum} {
	global coal_struct 
	return [lindex $coal_struct $coalnum]
}

proc coala_remove_coal {coal} {
	global coal_struct
	set i [lsearch $coal_struct	 $coal]
	if {$i != -1} {
		set coal_struct [lreplace $coal_struct $i $i]
	}
}

proc coala_get_coal_of_agent {agent} {
	global coal_struct 
	return [coala_get_coal_of_agent_in_struct $agent $coal_struct]
}

proc coala_get_coal_of_agent_in_struct {agent struct} {
	foreach coal $struct {
		if {[lsearch $coal $agent] != -1} {
			return $coal
		}
	}
}

proc coala_get_agent {agentnum} {
	global all_in_nego 
	return [lindex $all_in_nego $agentnum]
}

proc coala_get_agent_from_coal {agentnum coal} {
	return [lindex $coal $agentnum]
}



# ===== coala_calc_coal_value
# ===== calculate coalition value for given coalition
proc coala_calc_coal_value {coal} {
	global v_func infos_in_nego this_neg_ro algorithm

	set coal_value 0
	# ===== foreach agent in the current coal
	foreach agent $coal {
		# -- relevant own IUs are counted
		set ro 0.0
		# ===== foreach infounit(s) this agent has
		if [info exists infos_in_nego($agent)] {
			foreach entry $infos_in_nego($agent) {
				# ===== if this/these unit(s) are accessible for the coalition
				if [info_is_accessible $coal [lindex $entry 1] $agent] {
					# ===== test foreach agent in the coal
					foreach agent2 $coal {
						# ===== for v_a/v_g-CEs: if unit is available for this agent
						if {$v_func == 2} {
							if {[lsearch [lindex $entry 1] $agent2] == -1} {
								# ===== agent will not pay for something he cannot see (get)
								continue
							}
						}
						# ===== if infounit is relevant for this agent =>
						# ===== he will pay for it
						if {[lsearch [lindex $entry 2] $agent2] != -1} {
							# ===== individual price foreach info-unit
							# -- is relevant -> incr ro
							set ro [expr $ro + 1.0]
							set coal_value [expr $coal_value + [lindex $entry 0]]
						}
					}
				}
			}
		}
		# -- if two unary coals and agents don't know each other's ro value yet, set it
		if {$algorithm == "BSCA-RC" && [llength $coal] == 2} {
			set a1 [lindex $coal 0]
			set a2 [lindex $coal 1]
			if {$agent == $a1} {
				set index [list $a1 $a2]
			} else {
				set index [list $a2 $a1]
			}
			if {![info exists this_neg_ro($index)]} {
				set this_neg_ro($index) $ro
			}
		}
	}
	return $coal_value
}

# ===== coala_set_rel_infos_of_late_agent 
# ===== set the searching agent list of each agent for new coming agent 
proc coala_set_rel_infos_of_late_agent {agent} {
	global infos_in_nego rel_infos_in_nego cur_dyn_events_time

	foreach rel_info $rel_infos_in_nego($agent) { 
		set from_time [lindex $rel_info 2]
		if {$from_time > $cur_dyn_events_time} {
			continue
		}
		set i_agent [lindex $rel_info 0]
		set i_no [lindex $rel_info 1]
		set i_unit [lindex $infos_in_nego($i_agent) $i_no]
		set price [lindex $i_unit 0]
		set a_list [lindex $i_unit 1]
		set s_list [lindex $i_unit 2]
		# replace with new search list
		lappend s_list $agent
		set infos_in_nego($i_agent) [lreplace $infos_in_nego($i_agent) $i_no $i_no [list $price $a_list $s_list]]
	}
}


# ===== coala_delete_rel_infos_of_leaving_agent 
# ===== delete the rel_infos of leaving agent  
proc coala_delete_rel_infos_of_leaving_agent {agent} {
	global infos_in_nego rel_infos_in_nego

	foreach rel_info $rel_infos_in_nego($agent) { 
		# ==== correct searching agent list
		set i_agent [lindex $rel_info 0]
		set i_no [lindex $rel_info 1]
		set i_unit [lindex $infos_in_nego($i_agent) $i_no]
		set price [lindex $i_unit 0]
		set a_list [lindex $i_unit 1]
		set s_list [lindex $i_unit 2]
		# replace with new search list
		set j [lsearch $s_list $agent]
		if {$j != -1} {
			set s_list [lreplace $s_list $j $j]
			set infos_in_nego($i_agent) [lreplace $infos_in_nego($i_agent) $i_no $i_no [list $price $a_list $s_list]]
		}
	}	
}

# ======= coala_subtract_list 
# ======= calculates 'list1' - 'list2', then returns it
proc coala_subtract_list {list1 list2} {
	set new_list $list1
	foreach el2 $list2 {
		set i [lsearch $new_list $el2]
		set new_list [lreplace $new_list $i $i]
	}
	return $new_list
}

proc coala_add_network_time {num_packet} {
	global nego_start_time trans_time communication_time 

	set communication_time [expr $communication_time + $num_packet * $trans_time]
	set nego_start_time [expr $nego_start_time - $num_packet * $trans_time]
} 

proc coala_increase_num_building_coalition_nego {coal1 coal2} { 
	global num_building_coalition_nego
	
	foreach agent $coal1 {
		foreach agent1 $coal2 {
			incr num_building_coalition_nego([list $agent $agent1]) 
			incr num_building_coalition_nego([list $agent1 $agent]) 
		}
	}
}


proc coala_calc_rbsv {coal ref_bsv gain ref_founder_of} {
	global v_c
	upvar $ref_bsv bsv
	upvar $ref_founder_of founder_of

	if {[llength $coal] == 1} {
		set bsv($coal) [expr $v_c($coal) + $gain]
	} else {
		set C1 [lindex $founder_of($coal) 0]
		set C2 [lindex $founder_of($coal) 1]
		if ![info exists v_c($C1)] {
			set v_c($C1) [coala_calc_coal_value $C1]
		} 
		if ![info exists v_c($C2)] {
			set v_c($C2) [coala_calc_coal_value $C2]
		} 

		set alloc [expr 0.5 * ($v_c($coal) - $v_c($C1) - $v_c($C2) + $gain)]
	
		if {[llength $C1] == 1} {
			set bsv($C1) [expr $v_c($C1)+$alloc]
		} else {
			coala_calc_rbsv $C1 bsv $alloc founder_of
		}
	
		if {[llength $C2] == 1} {
			set bsv($C2) [expr $v_c($C2)+$alloc]
		} else {
			coala_calc_rbsv $C2 bsv $alloc founder_of
		}			
	}
}

proc coala_calc_coal_values_for_all_founders {coal ref_founder_of} {
	global v_c
	upvar $ref_founder_of founder_of

	set v_c($coal) [coala_calc_coal_value $coal]
	if {[llength $coal] == 1} {
		return
	} else {
		set C1 [lindex $founder_of($coal) 0]
		set C2 [lindex $founder_of($coal) 1]
		coala_calc_coal_values_for_all_founders $C1 founder_of
		coala_calc_coal_values_for_all_founders $C2 founder_of
	}
}

