# -----------------------------------------------------------------------------
# Copyright (c) 2003/4 by Matthias Klusch, Wooseok Park
# Computer Science Department,
# University of Saarland,
# Postfach 151150, 66041 Saarbr-Acken, Germany$)C
# 
# All rights reserved.
# No warranties will be given on any issues arising out of the use
# of this software product.
# Permission to use, copy, modify and distribute this software
# product for non-commercial purposes is hereby granted, provided
# that the above copyright notice appears in all copies and
# respective publications.
# All commercial trades with this product outside the University of
# Saarland without specific written prior permission are prohibited.
#-----------------------------------------------------------------------------

# -----------------------------------------------------------------------------
# Copyright (c) 1997/8 by Matthias Klusch, Torsten Vielhak
# Computer Science Department,
# Christian-Albrechts-University of Kiel,
# Olshausenstr. 40, 24118 Kiel, Germany
# 
# All rights reserved.
# No warranties will be given on any issues arising out of the use
# of this software product.
# Permission to use, copy, modify and distribute this software
# product for non-commercial purposes is hereby granted, provided
# that the above copyright notice appears in all copies and
# respective publications.
# All commercial trades with this product outside the CAU Kiel
# without specific written prior permission are prohibited.
#-----------------------------------------------------------------------------

# ====  coala_display_neg_coal_form
# ====  displays the coalition formations throughout the negotiation
# ====  adapted from coala only with small changes
proc coala_display_neg_coal_form {} {
	coala_read_formation_data
    # ===== make "new" special frame
    set w .analyze
    destroy $w.f1
    destroy $w.f2
    frame $w.f1
    pack $w.f1 -fill x -in $w.right

    # ===== clear canvas
    eval .analyze.canvas delete [.analyze.canvas find all]

    # ===== title for the canvas
    global c_font selected_neg
    .analyze.canvas create text 10 10 -text "Coalition Formation of Negotiation $selected_neg" -anchor w -font $c_font
    coala_bind_help .analyze.canvas "coala_shorthelp ana1_dg_coal_form.hlp"

    # ===== sizes for agents, ....
    set agent_width 70
    set agent_height 50
    set round_height 120
    set agent_space 25

    # ===== offsets
    set xoffset 90
    set yoffset 30

    # ===== colors
    global a_colors 

    global max_round coalition_struct payoff_vector agents_set participant termination_reason elapsed_time

    # ===== max. xpos of an agent (for canvas scrollregion)
    set maxxpos 0
	
	for {set r 0} {$r <= $max_round} {incr r} {
		set r_agents [lindex $agents_set $r]
		set r_coal_struct [lindex $coalition_struct $r]
		set r_payoff [lindex $payoff_vector $r]
		set xpos [expr 20 + $r * ($agent_space / 2) + $xoffset]

		# ===== display round number
		if {$r == 0} {
			$w.canvas create text 15 [expr $r * $round_height + $yoffset + $agent_height / 2] -text "Start Formation" -anchor w -font $c_font
		} else {
			$w.canvas create text 15 [expr $r * $round_height + $yoffset + $agent_height / 2] -text "After Round [expr $r]" -anchor w -font $c_font
			if {$r == $max_round} {
				$w.canvas create text 15 [expr $r * $round_height + $yoffset + $agent_height / 2 + 20] \
					-text "End Formation" -anchor w -font $c_font
				$w.canvas create text 15 [expr $r * $round_height + $yoffset + $agent_height / 2 + 40] \
					-text "Termination Reason : $termination_reason" -anchor w -font $c_font
				$w.canvas create text 15 [expr $r * $round_height + $yoffset + $agent_height / 2 + 60] \
					-text "Elapsed Time : $elapsed_time sec." -anchor w -font $c_font
			}
		}

		foreach coal $r_coal_struct {
	    	foreach agent [lsort $coal] {
				$w.canvas create rectangle $xpos [expr $r * $round_height + $yoffset] [expr $xpos + $agent_width] \
					 [expr $r * $round_height + $agent_height + $yoffset] -fill [lindex $a_colors [expr [lsearch $participant $agent] % [llength $a_colors]]]

				if {$r > 0 && [info exists previous_round($agent)]} {
					$w.canvas create line [expr $xpos + ($agent_width / 2)] [expr $r * $round_height + $yoffset] \
						[expr $previous_round($agent) + ($agent_width / 2)] [expr ($r - 1) *$round_height + $agent_height + $yoffset] \
						-arrow first -width 2
				}
		
				# ===== remember position
				set previous_round($agent) $xpos

				$w.canvas create text [expr $xpos + ($agent_width /2)] [expr $r * $round_height + 15 + $yoffset] \
					-text $agent -anchor c -justify center -font $c_font

				set payoff [lindex $r_payoff [lsearch $r_agents $agent]]
				if {$payoff == int($payoff)} {
				    $w.canvas create text [expr $xpos + ($agent_width /2)] [expr $r * $round_height + 30 + $yoffset] \
					-text $payoff -anchor c -justify center -font $c_font
				} else {
				    $w.canvas create text [expr $xpos + ($agent_width /2)] [expr $r * $round_height + 30 + $yoffset] \
					-text [format "%.4f" $payoff] -anchor c -justify center -font $c_font
				}
				incr xpos $agent_width
	    	}
		    incr xpos $agent_space
	    
		    # ===== remember max xpos
		    if {$xpos > $maxxpos} {
				set maxxpos $xpos
		    }
		}
    }
	coala_adjust_analyze_canvas
} 


# === read coalition structure and utilities
proc coala_read_formation_data {} {
	global simulation_name selected_neg
    global max_round coalition_struct payoff_vector agents_set participant 
	global termination_reason elapsed_time

	set coalition_struct {}
	set payoff_vector {}
	set agents_set {}
	set max_round 0
	set logfile [open "logs/$simulation_name\.$selected_neg\.log" r]
	gets $logfile line
	lappend agents_set [lreplace $line 0 0]
	gets $logfile line
	lappend coalition_struct [lreplace $line 0 0]
   	gets $logfile line
	lappend payoff_vector [lreplace $line 0 0]
	# ===== read formation after each round
	while {![eof $logfile]} {
   		gets $logfile line
		if {[lindex $line 0] == "negotiation_end"} {
			set termination_reason [lreplace $line 0 0]
			continue
		}

		if {[lindex $line 0] == "Participant"} {
			set participant [lreplace $line 0 0]
			break
		}
		if {[lindex $line 0] != "ragents"} {
			continue
		}
		lappend agents_set [lreplace $line 0 0]
   		gets $logfile line
		lappend coalition_struct [lreplace $line 0 0]
   		gets $logfile line
		lappend payoff_vector [lreplace $line 0 0]
		incr max_round
	}
	gets $logfile line
	set elapsed_time [lreplace $line 0 0]
	close $logfile
}