# -----------------------------------------------------------------------------
# Copyright (c) 2009 by Christine Walter, Matthias Klusch
# Saarland University and German Research Center for Artificial Intelligence,
# Saarbruecken, Germany
#
# All rights reserved. No warranties will be given on any issues arising out 
# of the use of this software product. Permission to use, copy, modify and 
# distribute this software product for non-commercial purposes is hereby 
# granted, provided that the above copyright notice appears in all copies and 
# respective publications.
# All commercial trades with this product without specific written prior 
# permission are prohibited.
# -----------------------------------------------------------------------------

proc  coala_display_opp_stats {} {
	
	set w .analyze
	destroy $w.f1

	# ===== make "new" special frame
	frame $w.f1
	pack $w.f1 -fill x -in $w.right
	
	# ====== frames
	frame $w.f1.f11 
	frame $w.f1.f12 
	frame $w.f1.f12_1 -relief ridge -bd 2
	frame $w.f1.f12_2 -relief ridge -bd 2

	pack $w.f1.f11 -side top -fill x -in $w.f1 
	pack $w.f1.f12 -side bottom -fill x -in $w.f1 
	pack $w.f1.f12_1 -side top -fill x -in $w.f1.f12 

	label $w.f1.l1 -text "Statistics for Opportunistic Behaviour"
	pack $w.f1.l1 -fill x -side top -in $w.f1.f11

	label $w.f1.l2 -text "Select Opportunistic Agent"
	scrollbar $w.f1.scroll1 -command "$w.f1.a_list yview"
	listbox $w.f1.a_list -yscroll "$w.f1.scroll1 set" \
		-relief raised -exportselection no -height 15 -width 16 -selectmode extended
	
	pack $w.f1.l2 -fill x -side top -in $w.f1.f12_1
	pack $w.f1.a_list -side left -fill both -expand 1 -in $w.f1.f12_1
	pack $w.f1.scroll1 -side right -fill y -in $w.f1.f12_1
    coala_bind_help $w.f1.l2 "coala_shorthelp ana2_lb_opp_stats.hlp"
    coala_bind_help $w.f1.a_list "coala_shorthelp ana2_lb_opp_stats.hlp"

	bind $w.f1.a_list <ButtonRelease-1> {
		coala_draw_opp_stats
	}

	# ===== fill lists
	global agents_list is_opp opp_participants

	set opp_participants {}
	# -- only insert opp agents
	foreach agent $agents_list {
		if $is_opp($agent) {
			$w.f1.a_list insert end $agent
			lappend opp_participants $agent
		}
    }

	# ===== adjust widths
	coala_check_list $w.f1.a_list 15
	# ===== select agent (by default : a_1)
	$w.f1.a_list selection set 0

	# ===== show utility 
	coala_draw_opp_stats
}




proc coala_draw_opp_stats {} {
	global c_font simulation_name opp_participants opt_sim

	# -- read data from logfile
	set stat_logfile [open "logs/$simulation_name\.opp_stats.log" r]
	set list_all {}

	# ===== clear canvas
	eval .analyze.canvas delete [.analyze.canvas find all]

	if {$opp_participants == {}} {
		.analyze.canvas create text 50 60 -text "There are no opportunistic agents." -font $c_font -anchor nw 
	} else {
	while {![eof $stat_logfile]} {
		gets $stat_logfile line
		if {$line == ""} {
			continue
		}
		set agent [lindex $line 0]
		set total_learning($agent) [lindex $line 1]
		set total_not_trusted($agent) [lindex $line 2]
		set latest_learning($agent) [lindex $line 3]
		set earliest_no_learning($agent) [lindex $line 4]
		set poss_behaviour($agent) [lindex $line 5]
		set i 6
		foreach action $poss_behaviour($agent) {
			set total_best_action([list $agent $action]) [lindex $line $i]
			incr i
		}
	}
	close $stat_logfile

	set agent [lindex $opp_participants [.analyze.f1.a_list curselection]]

	.analyze.canvas create text 10 30 -text "Statistics for opportunistic agent $agent" -font $c_font -anchor nw
    coala_bind_help .analyze.canvas "coala_shorthelp ana2_dg_opp_stats.hlp"

	set ld 15
	set y 60
	set xa 50
	set xb 350
	set xc 400

	set negs_total [lindex $opt_sim 0]

	.analyze.canvas create text $xa $y -text "# of negotiations spent learning:" -font $c_font -anchor nw 
	.analyze.canvas create text $xb $y -text "$total_learning($agent)" -font $c_font -anchor nw 
	.analyze.canvas create text $xc $y -text "[expr $total_learning($agent) * 100 / $negs_total] %" -font $c_font -anchor nw 
	set y [expr $y + $ld]
	.analyze.canvas create text $xa $y -text "# of negotiations spent rebuilding trust:" -font $c_font -anchor nw 
	.analyze.canvas create text $xb $y -text "$total_not_trusted($agent)" -font $c_font -anchor nw 
	.analyze.canvas create text $xc $y -text "[expr $total_not_trusted($agent) * 100 / $negs_total] %" -font $c_font -anchor nw 
	set y [expr $y + $ld]

	foreach action $poss_behaviour($agent) {
		.analyze.canvas create text $xa $y -text "# of negotiations with $action as best behaviour:" -font $c_font -anchor nw
		.analyze.canvas create text $xb $y -text "$total_best_action([list $agent $action])" -font $c_font -anchor nw 
		if {$total_best_action([list $agent $action]) > 0} {
			.analyze.canvas create text $xc $y -text "[expr $total_best_action([list $agent $action]) * 100 / $negs_total] %" -font $c_font -anchor nw 
		}
		set y [expr $y + $ld]
	}

	.analyze.canvas create text $xa $y -text "first negotiation without learning: negotiation $earliest_no_learning($agent)" -font $c_font -anchor nw 
	set y [expr $y + $ld]
	.analyze.canvas create text $xa $y -text "last negotiation with learning: negotiation $latest_learning($agent)" -font $c_font -anchor nw 


}
}



 
