# -----------------------------------------------------------------------------
# Copyright (c) 2009 by Christine Walter, Matthias Klusch
# Saarland University and German Research Center for Artificial Intelligence,
# Saarbruecken, Germany
#
# All rights reserved. No warranties will be given on any issues arising out 
# of the use of this software product. Permission to use, copy, modify and 
# distribute this software product for non-commercial purposes is hereby 
# granted, provided that the above copyright notice appears in all copies and 
# respective publications.
# All commercial trades with this product without specific written prior 
# permission are prohibited.
# -----------------------------------------------------------------------------

proc coala_display_opps_actions {} {

	set w .analyze
	destroy $w.f1

	# ===== make "new" special frame
	frame $w.f1
	pack $w.f1 -fill x -in $w.right
	
	# ====== frames
	frame $w.f1.f11 
	frame $w.f1.f12 
	frame $w.f1.f12_1 -relief ridge -bd 2
	frame $w.f1.f12_2 -relief ridge -bd 2

	pack $w.f1.f11 -side top -fill x -in $w.f1 
	pack $w.f1.f12 -side bottom -fill x -in $w.f1 
	pack $w.f1.f12_1 -side top -fill x -in $w.f1.f12 

	label $w.f1.l1 -text "Opportunist Actions"
	pack $w.f1.l1 -fill x -side top -in $w.f1.f11

	label $w.f1.l2 -text "Select Opportunistic Agents"
	scrollbar $w.f1.scroll1 -command "$w.f1.by_list yview"
	listbox $w.f1.by_list -yscroll "$w.f1.scroll1 set" \
		-relief raised -exportselection no -height 15 -width 16 -selectmode extended
	
	pack $w.f1.l2 -fill x -side top -in $w.f1.f12_1
	pack $w.f1.by_list -side left -fill both -expand 1 -in $w.f1.f12_1
	pack $w.f1.scroll1 -side right -fill y -in $w.f1.f12_1
    coala_bind_help $w.f1.l2 "coala_shorthelp ana2_lb_opp_actions.hlp"
    coala_bind_help $w.f1.by_list "coala_shorthelp ana2_lb_opp_actions.hlp"

	bind $w.f1.by_list <ButtonRelease-1> {
		coala_draw_actions
	}

	# ===== fill lists
	global agents_list is_opp opp_participants

	set opp_participants {}
	# -- only insert opp agents
	foreach agent $agents_list {
		if $is_opp($agent) {
			$w.f1.by_list insert end $agent
			lappend opp_participants $agent
		}
    }

	# ===== adjust widths
	coala_check_list $w.f1.by_list 15
	# ===== select agent (by default : a_1)
	$w.f1.by_list selection set 0

	# ===== show utility 
	coala_draw_actions
}

# ====== draws total utility 	
proc coala_draw_actions {} {
	global c_font agents_list simulation_name opp_participants

	# ===== clear canvas
	eval .analyze.canvas delete [.analyze.canvas find all]

	if {$opp_participants == {}} {
		.analyze.canvas create text 50 60 -text "There are no opportunistic agents." -font $c_font -anchor nw 
	} else {

	# ==== get max, min utility 
	set max_uvalue 0
	set i 0
	set all_actions_list {}
	set action_values {}
	foreach agent $opp_participants {
		set agent_actions {}
		set qlogfile [open "logs/$simulation_name\.q_tables_$agent\.log" r]
		while {![eof $qlogfile]} {
   			gets $qlogfile line
			if {[lindex $line 0] == "chosen_action"} {
				gets $qlogfile line
				set action [lindex $line 0]
				lappend agent_actions $action
				if  {$action > $max_uvalue} {
					set max_uvalue $action
				}
			}
		}
		close $qlogfile
		if {[lsearch [.analyze.f1.by_list curselection] $i] > -1} {
			lappend all_actions_list [concat $agent $agent_actions]
		}
		incr i
	}

	.analyze.canvas create text 10 20 -text "Actions Chosen by Opportunists" -font $c_font -anchor nw
    coala_bind_help .analyze.canvas "coala_shorthelp ana2_dg_opp_actions.hlp"

	# ===== draw axis
	set x_left	60
	set x_right 560
	set y_bottom 450
	set y_top 50

	.analyze.canvas create line $x_left $y_top $x_left $y_bottom -width 2
	.analyze.canvas create line $x_left $y_bottom $x_right $y_bottom -width 2
	# ===== adjust scale
	set max_int [expr int([expr ceil($max_uvalue)])]
	set min_int 0
	if {$max_int != 0 && $min_int != 0} {
		.analyze.canvas create line $x_left [expr $y_top + abs($max_int) * ($y_bottom - $y_top) / ($max_int - $min_int)] \
			$x_right [expr $y_top + abs($max_int) * ($y_bottom - $y_top) / ($max_int - $min_int)] -width 1 -fill #ffffff
	}

	if {$max_int == $min_int} {
		# ==== max_int = min_int = 0
		set int_diff 1
		set max_int 1
		set min_int 0
	} else {
		set int_diff [expr $max_int - $min_int]
	} 
		
	# ===== draw y-axis
	set j 0
	for {set i $max_int} {$i >= $min_int} {set i [expr $i - 1]} { 
		if {$i == [expr int($i)]} {
			.analyze.canvas create text [expr $x_left - 15] [expr $y_top + $j * ($y_bottom - $y_top) / $max_int] \
				-text [expr int($i)] -font $c_font
		} else {
			.analyze.canvas create text [expr $x_left - 15] [expr $y_top + $j * ($y_bottom - $y_top) / $max_int] \
				-text $i -font $c_font
		}
		.analyze.canvas create line $x_left [expr $y_top + $j * ($y_bottom - $y_top) / $max_int] [expr $x_left + 3] \
			[expr $y_top + $j * ($y_bottom - $y_top) / $max_int] -width 2
		incr j
	}

	# ===== draw x-axis
	global opt_sim
	set total_neg_no [lindex $opt_sim 0]
	set j 0
	for {set i 0} {$i < 11} {incr i} { 
		.analyze.canvas create text [expr $x_left + $i * ($x_right - $x_left) / 10] [expr $y_bottom + 10] \
			-text [expr $i * $total_neg_no / 10] -font $c_font
		.analyze.canvas create line [expr $x_left + $i * ($x_right - $x_left) / 10] $y_bottom \
			[expr $x_left + $i * ($x_right - $x_left) / 10] [expr $y_bottom - 3] -width 2
	}
	.analyze.canvas create text [expr ($x_left + $x_right) / 2 - 10] [expr $y_bottom + 25] -text "Negotiation Number" -font $c_font
	.analyze.canvas create text [expr $x_right + 80] 50 -text "Agent" -font $c_font


	# ====== draw graphs
	global agents_list simulation_name 

	set colorfile [open "rgb.txt" r]
	set cnt 1
	foreach agent_actions $all_actions_list {
		set agent [lindex $agent_actions 0]
		set actions_list [lreplace $agent_actions 0 0]
		# ===== set color
		gets $colorfile line
		if {$line != ""} {
			set r [lindex $line 0]
			set g [lindex $line 1]
			set b [lindex $line 2]
			set rgb [format "%02x%02x%02x" $r $g $b]
		} else {
			set rgb [format "%02x%02x%02x" 0 0 0]
		}

		set neg_no 0
		set uvalue [lindex $actions_list 0]
		set actions_list [lreplace $actions_list 0 0]
		set from_x $x_left
		set from_y [expr $y_bottom - ($uvalue - $min_int) * ($y_bottom - $y_top) / $int_diff] 

		foreach action $actions_list {
			if {$action == ""} {
				break
			}
			incr neg_no 
			set to_x [expr $x_left + $neg_no * ($x_right - $x_left) / $total_neg_no]
			set to_y [expr $y_bottom - ($action - $min_int) * ($y_bottom - $y_top) / $int_diff] 
			.analyze.canvas create line $from_x $from_y $to_x $to_y -width 2 -fill #$rgb -tags $agent
			set from_x $to_x
			set from_y $to_y
		}
		.analyze.canvas create text [expr $x_right + 80] [expr $cnt * 15 + 50] -text "$agent" -font $c_font
		.analyze.canvas create line [expr $x_right + 30] [expr $cnt * 15 + 50] [expr $x_right + 60] \
			[expr $cnt * 15 + 50] -width 2 -fill #$rgb -tags $agent
		incr cnt

		# ==== binding for thickening the graph by mouse clicking 
		.analyze.canvas bind $agent <Button-1> {
			set item_id [.analyze.canvas find withtag current]
			set tag_id [lindex [.analyze.canvas gettags $item_id] 0]
			.analyze.canvas itemconfigure $tag_id -width 6
		}
		.analyze.canvas bind $agent <ButtonRelease-1> {
			set item_id [.analyze.canvas find withtag current]
			set tag_id [lindex [.analyze.canvas gettags $item_id] 0]
			.analyze.canvas itemconfigure $tag_id -width 2
		}
	}
	close $colorfile
	# ====== adjust scrollregion
	coala_adjust_analyze_canvas
}
}

