# -----------------------------------------------------------------------------
# Copyright (c) 2009 by Christine Walter, Matthias Klusch
# Saarland University and German Research Center for Artificial Intelligence,
# Saarbruecken, Germany
#
# All rights reserved. No warranties will be given on any issues arising out 
# of the use of this software product. Permission to use, copy, modify and 
# distribute this software product for non-commercial purposes is hereby 
# granted, provided that the above copyright notice appears in all copies and 
# respective publications.
# All commercial trades with this product without specific written prior 
# permission are prohibited.
# -----------------------------------------------------------------------------


proc  coala_opp_display_q_tables {} {
    	global c_font
	global is_opp

    set w .analyze
    destroy $w.f1
    destroy $w.f2
    # ===== make "new" special frame
    frame $w.f1
    pack $w.f1 -side top -fill x -in $w.right
    
    # ====== frames
    frame $w.f1.f11 
    frame $w.f1.f12 -relief ridge -bd 2

	pack $w.f1.f11 -side top -fill x -in $w.f1 
	pack $w.f1.f12 -side top -fill y -in $w.f1 


    label $w.f1.l1 -text "Select Agent"
    pack $w.f1.l1 -fill x -side top -in $w.f1.f11

    scrollbar $w.f1.scroll1 -command "$w.f1.a_list yview"
    listbox $w.f1.a_list -yscroll "$w.f1.scroll1 set" \
		-relief raised -exportselection no -height 15 -width 16 
    
    pack $w.f1.a_list -side left -fill both -expand 1 -in $w.f1.f12
    pack $w.f1.scroll1 -side right -fill y -in $w.f1.f12
    coala_bind_help $w.f1.a_list "coala_shorthelp ana1_lb_q_tables.hlp"


	bind $w.f1.a_list <ButtonRelease-1> {
		coala_show_q_table
	}

    # ===== fill lists
	global simulation_name selected_neg
    global participant opp_participants

	set logfile [open "logs/$simulation_name\.$selected_neg\.log" r]
	while {![eof $logfile]} {
   		gets $logfile line
		if {[lindex $line 0] == "Participant"} {
			set participant [lreplace $line 0 0]
			break
		}
	}
	close $logfile
	set opp_participants {}
    foreach agent $participant {
	# -- only insert opp agents
		if $is_opp($agent) {
			$w.f1.a_list insert end $agent
			lappend opp_participants $agent
		}
    }

    # ===== adjust widths
    coala_check_list $w.f1.a_list 16
	# ===== select the first agent 
	$w.f1.a_list selection set 0

    # ===== show trust rating
    coala_show_q_table
}

proc coala_show_q_table {} {
	global c_font selected_neg simulation_name participant opp_participants
	global table_column_width table_row_height

	set agent [lindex $opp_participants [.analyze.f1.a_list curselection]]
	# -- read data from logfiles
	set q_file1 [open "logs/$simulation_name\.q_tables_$agent\.log" r]

	# -- find relevant negotiation

	gets $q_file1 line
	while {$line != "begin_neg $selected_neg"} {
		gets $q_file1 line
	}
	gets $q_file1 line
	gets $q_file1 c_action
	gets $q_file1 line
	set c_situation [lindex $line 0]
	set reason [lindex $line 1]
	
	switch $reason {
		"not_trusted" {set reason_text "to build trust"}
		"learning" {set reason_text "for learning"}
		"best_choice" {set reason_text "for maximum utility"}
	}

	gets $q_file1 profit
	gets $q_file1 line
	gets $q_file1 consequence


	# ===== clear canvas
	eval .analyze.canvas delete [.analyze.canvas find all]
    coala_bind_help .analyze.canvas "coala_shorthelp ana1_dg_q_tables.hlp"

global x_text y_text text_height table_column_width table_row_height
	set x_text 20
	set y_text 20
	set text_height 20
	set table_column_width 100
	set table_row_height 40





	.analyze.canvas create text $x_text $y_text -text "$agent chooses action $c_action in situation $c_situation $reason_text" \
		-anchor w -font $c_font

	set y_text [expr $y_text + $text_height]
	.analyze.canvas create text $x_text $y_text -text "profit made: $profit" \
		-anchor w -font $c_font

	set y_text [expr $y_text + $text_height]
	.analyze.canvas create text $x_text $y_text -text "trust loss caused: $consequence" \
		-anchor w -font $c_font

	set y_text [expr $y_text + $text_height * 2]

	# -- display Q table after negotiation


	.analyze.canvas create text $x_text $y_text -text "Q table of agent $agent after negotiation $selected_neg:" \
		-anchor w -font $c_font

	# -- table header


	set line ""

	while {$line != "q_table"} {
		gets $q_file1 line
	}

	gets $q_file1 line
	set actions_table [lindex $line 0]
	set situations_table [lindex $line 1]

	gets $q_file1 situation
	gets $q_file1 entries

	write_q_table_line_and_header $situation $entries
	while {$line != "end_q_table"} {
		gets $q_file1 situation
		if {$situation == "end_q_table"} {
			break
		}
		gets $q_file1 entries
		write_q_table_line $situation $entries
	}

	gets $q_file1 line
	set list_trust_loss {}
	while {$line != "end_trust_loss"} {
		gets $q_file1 line
		if {$line == "end_trust_loss"} {
			write_trust_loss $list_trust_loss
			break
		}
		set trust_loss [lindex $line 1]
		lappend list_trust_loss $trust_loss
	}


	close $q_file1

	# ===== find bounding box for all items in the canvas
	set xmin 100000
	set ymin 100000
	set xmax -100000
	set ymax -100000
    
	foreach id [.analyze.canvas find all] {
		if {[lindex [.analyze.canvas coords $id] 0] < $xmin} {
			set xmin [lindex [.analyze.canvas coords $id] 0]
		}		   
		if {[lindex [.analyze.canvas coords $id] 0] > $xmax} {
			set xmax [lindex [.analyze.canvas coords $id] 0]
		}
		if {[lindex [.analyze.canvas coords $id] 1] < $ymin} {
			set ymin [lindex [.analyze.canvas coords $id] 1]
		}
		if {[lindex [.analyze.canvas coords $id] 1] > $ymax} {
			set ymax [lindex [.analyze.canvas coords $id] 1]
		}
	}
    
	# ===== margins
	set ymax [expr $ymax + 30]

	# ===== adjust scrollregion
	coala_adjust_analyze_canvas


}

proc write_q_table_line_and_header {situation entries} {
	global table_column_width table_row_height x_text y_text x_table_left y_table_text c_font

	set x_table_left [expr $x_text + 5]
	set x_table_text $x_table_left
	set y_table_text [expr $y_text + $table_row_height]
	set yh_table_text $y_table_text


	.analyze.canvas create text $x_table_text $yh_table_text -text "entry" \
		-anchor w -font $c_font
	.analyze.canvas create text $x_table_text [expr $yh_table_text + 15] -text "(visited)" \
		-anchor w -font $c_font

	set y_table_text [expr $y_table_text + $table_row_height]
	.analyze.canvas create text $x_table_text $y_table_text -text "situation $situation" \
		-anchor w -font $c_font

	foreach entry $entries {
		set action [lindex $entry 0]
		set value [lindex $entry 1]
		set visited [lindex $entry 2]

		set x_table_text [expr $x_table_text + $table_column_width]

		.analyze.canvas create text $x_table_text $yh_table_text -text "action $action" \
			-anchor w -font $c_font

		.analyze.canvas create text $x_table_text $y_table_text -text "$value" \
			-anchor w -font $c_font
		.analyze.canvas create text $x_table_text [expr $y_table_text + 15] -text "($visited)" \
			-anchor w -font $c_font
		
	}
	# -- horizontal line above table
	q_table_horizontal_line $yh_table_text $x_table_text
	# -- horizontal line below table header
	q_table_horizontal_line $y_table_text $x_table_text
	# -- horizontal line below first table row
	q_table_horizontal_line [expr $y_table_text + $table_row_height] $x_table_text
	# -- vertical lines across header
	q_table_vertical_lines $yh_table_text $x_table_text
	# -- vertical lines across first table row
	q_table_vertical_lines $y_table_text $x_table_text
}



proc write_q_table_line {situation entries} {
	global table_column_width table_row_height x_text y_text c_font x_table_left y_table_text

	set y_table_text [expr $y_table_text + $table_row_height]
	set x_table_text $x_table_left

	.analyze.canvas create text $x_table_text $y_table_text -text "situation $situation" \
		-anchor w -font $c_font

	foreach entry $entries {
		set value [lindex $entry 1]
		set visited [lindex $entry 2]

		set x_table_text [expr $x_table_text + $table_column_width]

		.analyze.canvas create text $x_table_text $y_table_text -text "$value" \
			-anchor w -font $c_font
		.analyze.canvas create text $x_table_text [expr $y_table_text + 15] -text "($visited)" \
			-anchor w -font $c_font
		
	}
	# -- horizontal line below table row
	q_table_horizontal_line [expr $y_table_text + $table_row_height] $x_table_text
	# -- vertical lines across table row
	q_table_vertical_lines $y_table_text $x_table_text
}


proc write_trust_loss {trust_losses} {
	global table_column_width table_row_height x_text y_text c_font x_table_left y_table_text

	set y_table_text [expr $y_table_text + $table_row_height]
	set x_table_text $x_table_left

	.analyze.canvas create text $x_table_text $y_table_text -text "trust loss" \
		-anchor w -font $c_font

	foreach entry $trust_losses {

		set x_table_text [expr $x_table_text + $table_column_width]

		.analyze.canvas create text $x_table_text $y_table_text -text "[format "%.3f" $entry]" \
			-anchor w -font $c_font
		
	}
	# -- double horizontal line above trust loss row
	q_table_horizontal_line [expr $y_table_text + 2] $x_table_text
	# -- vertical lines across trust loss row
	q_table_vertical_lines $y_table_text $x_table_text
	# -- horizontal line below  trust loss row
	q_table_horizontal_line [expr $y_table_text + $table_row_height] $x_table_text
}

proc q_table_vertical_lines {y x} {
global table_column_width table_row_height

	set y [expr $y -12]
	set x_end [expr $x + $table_column_width]
	set x 20

	while {$x <= $x_end} {
		.analyze.canvas create line $x $y $x [expr $y + $table_row_height]
		set x [expr $x + $table_column_width]
	}
}

# -- horizontal line above/below table row
proc q_table_horizontal_line {y x} {
global table_column_width
	set y [expr $y -12]
	set x [expr $x + $table_column_width - 5]

	.analyze.canvas create line 20 $y $x $y 
}
