# -----------------------------------------------------------------------------
# Copyright (c) 2009 by Christine Walter, Matthias Klusch
# Saarland University and German Research Center for Artificial Intelligence,
# Saarbruecken, Germany
#
# All rights reserved. No warranties will be given on any issues arising out 
# of the use of this software product. Permission to use, copy, modify and 
# distribute this software product for non-commercial purposes is hereby 
# granted, provided that the above copyright notice appears in all copies and 
# respective publications.
# All commercial trades with this product without specific written prior 
# permission are prohibited.
# -----------------------------------------------------------------------------



proc coala_display_sim_average_utility {} {

	set w .analyze
	destroy $w.f1

	# ===== make "new" special frame
	frame $w.f1
	pack $w.f1 -fill x -in $w.right
	
	# ====== frames
	frame $w.f1.f11 
	frame $w.f1.f12 
	frame $w.f1.f12_1 -relief ridge -bd 2
	frame $w.f1.f12_2 -relief ridge -bd 2

	pack $w.f1.f11 -side top -fill x -in $w.f1 
	pack $w.f1.f12 -side bottom -fill x -in $w.f1 
	pack $w.f1.f12_1 -side top -fill x -in $w.f1.f12 

	label $w.f1.l1 -text "Development of \n Average Utility"
	pack $w.f1.l1 -fill x -side top -in $w.f1.f11


	# ===== show utility 
	coala_draw_average_utility
}

# ====== draws total utility 	
proc coala_draw_average_utility {} {
	global c_font agents_list simulation_name

	# ==== get max, min utility 
	set max_uvalue 0
	set i 0
	set all_utility_list {}
	foreach agent $agents_list {
		set logfile [open "logs/$simulation_name\.utility$agent\.log" r]
		gets $logfile line
		set uvalue [lindex $line [expr [llength $line] - 1]]
		if {$max_uvalue < $uvalue} {
			set max_uvalue $uvalue
		}
		lappend all_utility_list [concat $agent $line]
		close $logfile
		incr i
	}

	# -- compute average utilities
	global is_opp act_mal
	

	set coop_utilities {}
	set mal_utilities {}
	set opp_utilities {}
	set mal_opp_utilities {}
	set coop_agents 0
	set mal_agents 0
	set opp_agents 0
	set mal_opp_agents 0
	foreach agent_util $all_utility_list {
		set agent [lindex $agent_util 0]
		set utility_list [lreplace $agent_util 0 0]
		if {$is_opp($agent) == 1} {
			if {$act_mal($agent)} {
				if {$mal_opp_agents != 0} {
					set i 0
					foreach utility $utility_list {
						set mal_opp_utilities [lreplace $mal_opp_utilities $i $i [expr [lindex $mal_opp_utilities $i] + $utility]]
						incr i
					}
				} else {
					set mal_opp_utilities $utility_list
				}
				incr mal_opp_agents
			} else {
				if {$opp_agents != 0} {
					set i 0
					foreach utility $utility_list {
						set opp_utilities [lreplace $opp_utilities $i $i [expr [lindex $opp_utilities $i] + $utility]]
						incr i
					}
				} else {
					set opp_utilities $utility_list
				}
				incr opp_agents
			}
		} else {
			if {$act_mal($agent)} {
				if {$mal_agents != 0} {
					set i 0
					foreach utility $utility_list {
						set mal_utilities [lreplace $mal_utilities $i $i [expr [lindex $mal_utilities $i] + $utility]]
						incr i
					}
				} else {
					set mal_utilities $utility_list
				}
				incr mal_agents
			} else {
				if {$coop_agents != 0} {
					set i 0
					foreach utility $utility_list {
						set coop_utilities [lreplace $coop_utilities $i $i [expr [lindex $coop_utilities $i] + $utility]]
						incr i
					}
				} else {
					set coop_utilities $utility_list
				}
				incr coop_agents
			}
		}
	}
	# -- compute averages
	set averages_list {}

	if {$coop_agents != 0} {
		set i 0
		foreach utility $coop_utilities {
			set coop_utilities [lreplace $coop_utilities $i $i [format "%.1f" [expr $utility / $coop_agents]]]
			incr i
		}
		lappend averages_list [concat "Cooperative" $coop_utilities]
	}
	if {$mal_agents != 0} {
		set i 0
		foreach utility $mal_utilities {
			set mal_utilities [lreplace $mal_utilities $i $i [format "%.1f" [expr $utility / $mal_agents]]]
			incr i
		}
		lappend averages_list [concat "Malicious" $mal_utilities]
	}
	if {$opp_agents != 0} {
		set i 0
		foreach utility $opp_utilities {
			set opp_utilities [lreplace $opp_utilities $i $i [format "%.1f" [expr $utility / $opp_agents]]]
			incr i
		}
		lappend averages_list [concat "just Opp." $opp_utilities]
	}
	if {$mal_opp_agents != 0} {
		set i 0
		foreach utility $mal_opp_utilities {
			set mal_opp_utilities [lreplace $mal_opp_utilities $i $i [format "%.1f" [expr $utility / $mal_opp_agents]]]
			incr i
		}
		lappend averages_list [concat "Opportunists" $mal_opp_utilities]
	}



	# ===== clear canvas
	eval .analyze.canvas delete [.analyze.canvas find all]
	.analyze.canvas create text 10 20 -text "Development of Average Utilities" -font $c_font -anchor nw
    coala_bind_help .analyze.canvas "coala_shorthelp ana2_dg_average_util.hlp"

	# ===== draw axis
	set x_left	60
	set x_right 560
	set y_bottom 450
	set y_top 50

	.analyze.canvas create line $x_left $y_top $x_left $y_bottom -width 2
	.analyze.canvas create line $x_left $y_bottom $x_right $y_bottom -width 2
	# ===== adjust scale
	set max_int [expr int([expr ceil($max_uvalue)])]
	if {$max_int > 10} {
		set marg [expr 10 - ($max_int % 10)]
		set max_int [expr $max_int + $marg]
	}
	set min_int 0
	if {$max_int != 0 && $min_int != 0} {
		.analyze.canvas create line $x_left [expr $y_top + abs($max_int) * ($y_bottom - $y_top) / ($max_int - $min_int)] \
			$x_right [expr $y_top + abs($max_int) * ($y_bottom - $y_top) / ($max_int - $min_int)] -width 1 -fill #ffffff
	}

	if {$max_int == $min_int} {
		# ==== max_int = min_int = 0
		set int_diff 1
		set max_int 1
		set min_int 0
	} else {
		set int_diff [expr $max_int - $min_int]
	} 
		
	# ===== draw y-axis
	set j 0
	for {set i $max_int} {$i >= $min_int} {set i [expr $i - $int_diff / 10.0]} { 
		if {$i == [expr int($i)]} {
			.analyze.canvas create text [expr $x_left - 15] [expr $y_top + $j * ($y_bottom - $y_top) / 10] \
				-text [expr int($i)] -font $c_font
		} else {
			.analyze.canvas create text [expr $x_left - 15] [expr $y_top + $j * ($y_bottom - $y_top) / 10] \
				-text $i -font $c_font
		}
		.analyze.canvas create line $x_left [expr $y_top + $j * ($y_bottom - $y_top) / 10] [expr $x_left + 3] \
			[expr $y_top + $j * ($y_bottom - $y_top) / 10] -width 2
		incr j
	}


	# ===== draw x-axis
	global opt_sim
	set total_neg_no [lindex $opt_sim 0]
	set j 0
	for {set i 0} {$i < 11} {incr i} { 
		.analyze.canvas create text [expr $x_left + $i * ($x_right - $x_left) / 10] [expr $y_bottom + 10] \
			-text [expr $i * $total_neg_no / 10] -font $c_font
		.analyze.canvas create line [expr $x_left + $i * ($x_right - $x_left) / 10] $y_bottom \
			[expr $x_left + $i * ($x_right - $x_left) / 10] [expr $y_bottom - 3] -width 2
	}
	.analyze.canvas create text [expr ($x_left + $x_right) / 2 - 10] [expr $y_bottom + 25] -text "Negotiation Number" -font $c_font
	.analyze.canvas create text [expr $x_right + 80] 50 -text "Agents" -font $c_font
	.analyze.canvas create text [expr $x_right + 140] 50 -text "Total" -font $c_font


	# ====== draw graphs
	global agents_list simulation_name 

	set colorfile [open "rgb.txt" r]
	set cnt 1
	set all_util 0
	foreach agent_util $averages_list {
		set agent [lindex $agent_util 0]
		set utility_list [lreplace $agent_util 0 0]
		# ===== set color
		gets $colorfile line
		if {$line != ""} {
			set r [lindex $line 0]
			set g [lindex $line 1]
			set b [lindex $line 2]
			set rgb [format "%02x%02x%02x" $r $g $b]
		} else {
			set rgb [format "%02x%02x%02x" 0 0 0]
		}

		set neg_no 0
		set uvalue [lindex $utility_list 0]
		set utility_list [lreplace $utility_list 0 0]
		set from_x $x_left
		set from_y [expr $y_bottom - ($uvalue - $min_int) * ($y_bottom - $y_top) / $int_diff] 

		foreach utility $utility_list {
			if {$utility == ""} {
				break
			}
			incr neg_no 
			set to_x [expr $x_left + $neg_no * ($x_right - $x_left) / $total_neg_no]
			set to_y [expr $y_bottom - ($utility - $min_int) * ($y_bottom - $y_top) / $int_diff] 
			.analyze.canvas create line $from_x $from_y $to_x $to_y -width 2 -fill #$rgb -tags $agent
			set from_x $to_x
			set from_y $to_y
		}
		set mvalue [lindex $utility_list [expr [llength $utility_list] - 1]]
		set all_util [expr $all_util + $mvalue]
		.analyze.canvas create text [expr $x_right + 80] [expr $cnt * 15 + 50] -text "$agent: " -font $c_font
		.analyze.canvas create text [expr $x_right + 140] [expr $cnt * 15 + 50] -text $mvalue -font $c_font
		.analyze.canvas create line [expr $x_right + 0] [expr $cnt * 15 + 50] [expr $x_right + 30] \
			[expr $cnt * 15 + 50] -width 2 -fill #$rgb -tags $agent
		incr cnt

		# ==== binding for thickening the graph by mouse clicking 
		.analyze.canvas bind $agent <Button-1> {
			set item_id [.analyze.canvas find withtag current]
			set tag_id [lindex [.analyze.canvas gettags $item_id] 0]
			.analyze.canvas itemconfigure $tag_id -width 6
		}
		.analyze.canvas bind $agent <ButtonRelease-1> {
			set item_id [.analyze.canvas find withtag current]
			set tag_id [lindex [.analyze.canvas gettags $item_id] 0]
			.analyze.canvas itemconfigure $tag_id -width 2
		}
	}
	close $colorfile
	# ====== adjust scrollregion
	coala_adjust_analyze_canvas
}
