# -----------------------------------------------------------------------------
# Copyright (c) 2009 by Christine Walter, Matthias Klusch
# Saarland University and German Research Center for Artificial Intelligence,
# Saarbruecken, Germany
#
# All rights reserved. No warranties will be given on any issues arising out 
# of the use of this software product. Permission to use, copy, modify and 
# distribute this software product for non-commercial purposes is hereby 
# granted, provided that the above copyright notice appears in all copies and 
# respective publications.
# All commercial trades with this product without specific written prior 
# permission are prohibited.
# -----------------------------------------------------------------------------

# -----------------------------------------------------------------------------
# Copyright (c) 2003/4 by Matthias Klusch, Wooseok Park
# Computer Science Department,
# University of Saarland,
# Postfach 151150, 66041 Saarbr-A|cken, Germany$)C
# 
# All rights reserved.
# No warranties will be given on any issues arising out of the use
# of this software product.
# Permission to use, copy, modify and distribute this software
# product for non-commercial purposes is hereby granted, provided
# that the above copyright notice appears in all copies and
# respective publications.
# All commercial trades with this product outside the University of
# Saarland without specific written prior permission are prohibited.
#-----------------------------------------------------------------------------



# ===== coala_fill_description.tcl
# ===== fill description_text widget according to selected simulation 
proc coala_fill_description {} {
	global agents_list	is_opp	act_mal

    .description_text configure -state normal
	.description_text delete 1.0 end

	.description_text insert end "<Agents in Simulation>\n"
	.description_text insert end "$agents_list\n"
	set coop_agents {}
	set mal_agents {}
	set opp_agents {}
	foreach agent $agents_list {
		if {$is_opp($agent)} {
			lappend opp_agents $agent
		} else {
			if {$act_mal($agent)} {
				lappend mal_agents $agent
			} else {
				lappend coop_agents $agent
			}
		}
	}
	if {[llength $coop_agents] > 0} {
	.description_text insert end "\n*Cooperative Agents: $coop_agents\n"
	} else {
	.description_text insert end "*Cooperative Agents: none\n"
	}
	if {[llength $opp_agents] > 0} {
	.description_text insert end "*Opportunistic Agents: $opp_agents\n"
	} else {
	.description_text insert end "*Opportunistic Agents: none\n"
	}
	if {[llength $mal_agents] > 0} {
	.description_text insert end "*Malicious Agents: $mal_agents\n"
	} else {
	.description_text insert end "*Malicious Agents: none\n"
	}

	.description_text insert end "\n<Optionsettings for Simulation>\n"
	.description_text insert end "------------------------------------------------\n"

	global opt_sim
	.description_text insert end "*Number of Negotiations : [lindex $opt_sim 0]\n"
	.description_text insert end "*Max. Time for One Negotiation (sec) : [lindex $opt_sim 1]\n"
	.description_text insert end "*Packet Sending Time (millisec) : [lindex $opt_sim 2]\n"
	switch [lindex $opt_sim 3] {
		1 {.description_text insert end "*Coalition Algorithm : BSCA-D\n"}
		2 {.description_text insert end "*Coalition Algorithm : BSCA-T\n"}
		3 {.description_text insert end "*Coalition Algorithm : BSCA-T*\n"}
		4 {.description_text insert end "*Coalition Algorithm : BSCA-TR\n"}
		5 {.description_text insert end "*Coalition Algorithm : BSCA-TR*\n"}
		6 {.description_text insert end "*Coalition Algorithm : BSCA-Opp\n"}
		7 {.description_text insert end "*Coalition Algorithm : BSCA-RC\n"}
		default {}
	}
	.description_text insert end "*Max. Number of Proposals of a Coalition : [lindex $opt_sim 4]\n"

	if {[lindex $opt_sim 5] == 1} {
		.description_text insert end "*Coalition Value Function : General Valuefunction\n"
	} else {
		.description_text insert end "*Coalition Value Function : Social Valuefunction\n"
	}

	if {[lindex $opt_sim 6] == 1} {
		.description_text insert end "*Random Numbers for Simulation : Only New Generating\n"
	} elseif {[lindex $opt_sim 6] == 2} {
		.description_text insert end "*Random Numbers for Simulation : New Generating and Save\n"
		.description_text insert end "*Filename to Save : [lindex $opt_sim 7]\n"
	} else {
		.description_text insert end "*Random Numbers for Simulation : Use Random Number File\n"
		.description_text insert end "*Random Number File : [lindex $opt_sim 7]\n"
	}
	.description_text insert end "\n"

	global infos
	foreach agent $agents_list {
		.description_text insert end "Information-Units of Agent $agent\n"
		.description_text insert end "------------------------------------------------\n"
		if {[llength $infos($agent)] == 0} {
			.description_text insert end "none\n"
		} else {
			set i 1 
			foreach entry $infos($agent) {
				set price [lindex $entry 0]
				set access [lindex $entry 1]
				set j [lsearch $access $agent]
				set access [lreplace $access $j $j]
				.description_text insert end "*Info-Unit$i\n"
				.description_text insert end "   Price : $price\n"
				.description_text insert end "   Agents with Accessing Right : $access\n"
				incr i  
			}
		}
		.description_text insert end "\n"
	}


	foreach agent  $agents_list {
		.description_text insert end "Accessible Information-Units for Agent $agent\n"
		.description_text insert end "--------------------------------------------------------------\n"

		set access_list {}
		# ==== set list of all accessible info-units 
		foreach iagent $agents_list {
			if {$iagent == $agent} {
				continue
			}
			set total_infos [llength $infos($iagent)]
			for {set i 0} {$i < $total_infos} {incr i} {
				set info_unit [lindex $infos($iagent) $i]
				set price [lindex $info_unit 0]
				set access [lindex $info_unit 1]
				if {[lsearch $access $agent] > -1} {
					lappend access_list [list [expr $i + 1] $iagent $price]
				}
			}
		}
		if {[llength $access_list] == 0} {
			.description_text insert end "none\n"
		} else {
			foreach acc_info $access_list {			
				set index [lindex $acc_info 0]
				set iagent [lindex $acc_info 1]
				set price [lindex $acc_info 2]
				.description_text insert end "Info-Unit $index of $iagent, Price : $price\n"
			}
		}
		.description_text insert end "\n"
	}

	.description_text insert end "\n<Optionsettings for Agents>\n"
	.description_text insert end "----------------------------------------\n"


	global opt_agent agents_list
	global opt_agent_text

	foreach agent  $agents_list {
		.description_text insert end "For Agent $agent\n"
		.description_text insert end "-----------------------\n"
		set i 0  
		catch {
			foreach entry $opt_agent($agent) {
				.description_text insert end "*[lindex $opt_agent_text $i] $entry\n" 
				incr i  
			}
		}
		.description_text insert end "\n"
	}
    .description_text configure -state disabled
}
