# -----------------------------------------------------------------------------
# Copyright (c) 2009 by Christine Walter, Matthias Klusch
# Saarland University and German Research Center for Artificial Intelligence,
# Saarbruecken, Germany
#
# All rights reserved. No warranties will be given on any issues arising out 
# of the use of this software product. Permission to use, copy, modify and 
# distribute this software product for non-commercial purposes is hereby 
# granted, provided that the above copyright notice appears in all copies and 
# respective publications.
# All commercial trades with this product without specific written prior 
# permission are prohibited.
# -----------------------------------------------------------------------------


# -- adds new experience to knowledge
proc coala_opp_add_experience {agent profit} {
	global game_situation agents_list total_utility_change num_building_coalition_nego game_trust_sum neg_number
	global behaviour q_table_entry q_table_index q_visited q_tables_file time_to_be_opportunist trust_loss
	global actions_tried not_trusted simulation_name

	# -- compute new situation (new game_trust_sum)

	set q_tables_file($agent) [open "logs/$simulation_name\.q_tables_$agent\.log" a]
	coala_puts $q_tables_file($agent) "$profit"

	set real_situation $game_trust_sum($agent)
	set situation $game_situation($agent)
	set game_trust_sum($agent) 0
	set not_trusted($agent) 0
	set trust_threshhold 0
	# -- if opp is not trusted by at least one other agent that is not untrustworthy itself, set var not_trusted to 1
	foreach agent2 $agents_list {
		if {$agent == $agent2} {
			continue
		}
		if {$num_building_coalition_nego($agent2 $agent) != 0} {
		set sc_agent [expr $total_utility_change($agent2 $agent) / $num_building_coalition_nego($agent2 $agent)]
		set sc_self_agent [expr $total_utility_change($agent $agent2) / $num_building_coalition_nego($agent2 $agent)]
		} else {
			set sc_agent 0
			set sc_self_agent 0
		}
		set game_trust_sum($agent) [expr $game_trust_sum($agent) + $sc_agent]
		# -- if not trusted by a trustworthy agent, rebuild trust before cheating again
		if {$sc_agent < $trust_threshhold && $sc_self_agent > 0} {
			set not_trusted($agent) 1
		}
	}


	# -- new situation for next negotiation, if situation is new, init new entries (if past initial stage yet)
	set game_situation($agent) [coala_round $game_trust_sum($agent)]
	set real_new_situation $game_trust_sum($agent)
	set new_situation $game_situation($agent)
	# -- if entry exists, update it
	set index [concat $agent $situation $behaviour($agent)]
	if {[lsearch $q_table_index($agent) $situation] != -1} {
		set alpha [expr 1.0 / (1.0 + $q_visited($index))]
		set q_table_entry($index) [expr (1.0 - $alpha) * $q_table_entry($index) + \
			$alpha * ($profit + [coala_max_q_entry $agent $new_situation])]
		set q_visited($index) [expr $q_visited($index) + 1]
	} else {
	# -- in the beginning no decide_behaviour, so have to init new entries here (first neg only)
		coala_init_q_table_entry $agent $situation $behaviour($agent) $profit $new_situation
	}
	set consequence [expr $real_situation - $real_new_situation]
	coala_puts $q_tables_file($agent) "caused_trust_loss"
	coala_puts $q_tables_file($agent) "$consequence"
	close $q_tables_file($agent)
	set index2 [concat $agent $behaviour($agent)]
	set tried [lindex $actions_tried($agent) $behaviour($agent)]
	if {$tried != 0} {
		set trust_loss($index2) [expr ($trust_loss($index2) * $tried + $consequence) / ($tried + 1)]
	} else {
		set trust_loss($index2) $consequence
	}
	set actions_tried($agent) [lreplace $actions_tried($agent) $behaviour($agent) $behaviour($agent) [expr $tried + 1]]
}
 
# -- in the beginning no decide_behaviour, so have to init new entries and add situation to index-list
proc coala_init_q_table_entry {agent situation action profit new_situation} {
	global q_table_entry q_table_index q_tables_file q_visited 
	
	set index [concat $agent $situation $action]
	set q_table_entry($index) [expr $profit + [coala_max_q_entry $agent $new_situation]]
	set q_visited($index) 1
	set q_table_index($agent) [lsort -integer [lappend q_table_index($agent) $situation]]
}

# -- find and return the max value for any action in situation, if info !exists, init it or return 0
proc coala_max_q_entry {agent situation} {
	global poss_behaviour q_table_entry time_to_be_opportunist q_visited q_table_index
	global q_tables_file

	set max_val 0
	foreach action $poss_behaviour($agent) {
		set index [concat $agent $situation $action]
		# -- if first stage of game, only entries for action 0 exist
		if {![info exists q_table_entry($index)]} {
			continue
		}
		if {$q_table_entry($index) > $max_val && [coala_opp_action_allowed $agent $situation $action]} {
			set max_val $q_table_entry($index)
		}
	}
	return $max_val
}

proc coala_round {val} {
	if {$val >= 0.5} {
		return [coala_round2 $val 1]
	} else {
		if {$val < -0.5} {
			return [expr 0 - [coala_round2 [expr 0 - $val] 1]]
		} else {
			return 0
		}
	}
}

proc coala_round2 {val i} {
	if {$val < [expr $i + 0.5] && $val >= [expr $i - 0.5]} {
		return $i
	} else {
		incr i
		coala_round2 $val $i
	}
}


