# -----------------------------------------------------------------------------
# Copyright (c) 2009 by Christine Walter, Matthias Klusch
# Saarland University and German Research Center for Artificial Intelligence,
# Saarbruecken, Germany
#
# All rights reserved. No warranties will be given on any issues arising out 
# of the use of this software product. Permission to use, copy, modify and 
# distribute this software product for non-commercial purposes is hereby 
# granted, provided that the above copyright notice appears in all copies and 
# respective publications.
# All commercial trades with this product without specific written prior 
# permission are prohibited.
# -----------------------------------------------------------------------------

# -- sets behaviour(agent) to the chosen number of trick info units

proc coala_opp_decide_behaviour {agent situation} {
	global trick_rel_info behaviour game_situation q_tables_file q_table_entry total_opp_actions simulation_name
	global total_good_actions done_learning total_opp_best_choice total_good_best_choice not_trusted
	global neg_number
	global total_not_trusted total_learning total_best_action
	 
	set reason "learning"
	if {[coala_time_to_be_opportunist $agent]} {
		set done_learning 0
		set behaviour($agent) [coala_find_best_opp_behaviour $agent $situation]
		if {$behaviour($agent) != 0} {
			incr total_opp_actions($agent)
			if $done_learning {
				incr total_opp_best_choice($agent)
				incr total_best_action([list $agent $behaviour($agent)])
			} else {
				incr total_learning($agent)
			}
		} else {
			incr total_good_actions($agent)
			if $not_trusted($agent) {
				incr total_not_trusted($agent)
			} else {
				if $done_learning {
					incr total_good_best_choice($agent)
					incr total_best_action([list $agent 0])
				} else {
					incr total_learning($agent)
				}
			}
		}
		if $done_learning {
			set reason "best_choice"
		}
		if $not_trusted($agent) {
			set reason "not_trusted"
		}
	} else {
		incr total_good_actions($agent)
		incr total_learning($agent)
		set behaviour($agent) 0
	}

	set q_tables_file($agent) [open "logs/$simulation_name\.q_tables_$agent\.log" a]
	coala_puts $q_tables_file($agent) "begin_neg $neg_number"
	coala_puts $q_tables_file($agent) "chosen_action"
	coala_puts $q_tables_file($agent) "$behaviour($agent)"
	coala_puts $q_tables_file($agent) "$game_situation($agent) $reason"
	close $q_tables_file($agent)

	return $behaviour($agent)
}

# -- checks if initial time of acting good is over yet

proc coala_time_to_be_opportunist {agent} {
	global agents_list num_building_coalition_nego init_trial_time time_to_be_opportunist q_tables_file

	if {!$time_to_be_opportunist($agent)} {
		set number_agents [llength $agents_list]
		set counter 0
		foreach o_agent $agents_list {
			if {$o_agent != $agent && $num_building_coalition_nego($agent $o_agent) > 0} {
				incr counter 1
			}
		}
		if {$counter > [expr 0.01 * $number_agents]} {
			set time_to_be_opportunist($agent) 1
			coala_init_q_table_entries_tricks $agent
			return 1
		} else {
			return 0
		}
	} else {
		return 1
	}
}

# -- return behaviour with max expected reward, actions that make trust_sum in game negative are strictly forbidden

proc coala_find_best_opp_behaviour {agent situation} {
	global poss_behaviour q_table_entry q_table_index q_visited
	global q_tables_file done_learning neg_number earliest_no_learning latest_learning

	# -- if new situation, init entries first
	if {[lsearch $q_table_index($agent) $situation] == -1} {
		coala_init_q_table_entries $agent $situation
	}
	# -- default choice is good behaviour
	set best_choice 0
	set max_val -1

	# -- try every allowed action at least n times
	set tried_actions_list {}
	set try_actions_list {}
	set first_try_actions_list {}
	# -- first try everything m times, then n times, then choose best allowed action
	set n 3
	set m 1
	foreach action $poss_behaviour($agent) {
		set index [concat $agent $situation $action]
		if [coala_opp_action_allowed $agent $situation $action] {
			lappend tried_actions_list $action

			if {$q_visited($index) < $n} {
				lappend try_actions_list $action
				if {$q_visited($index) < $m} {
					lappend first_try_actions_list $action
				}
			}
		}
	}
	# -- if all allowed actions have been tried n times, free choice
	if {$first_try_actions_list == {}} {
		if {$try_actions_list == {}} {
			set done_learning 1
			set actions_list $tried_actions_list
			if {$earliest_no_learning($agent) == 0} {
				set earliest_no_learning($agent) $neg_number
			}
		} else {
			set actions_list $try_actions_list
			set latest_learning($agent) $neg_number
			}
	} else {
		set actions_list $first_try_actions_list
		set latest_learning($agent) $neg_number
	}
	
	foreach action $actions_list {
		set index [concat $agent $situation $action]
		if {$q_table_entry($index) > $max_val} {
			set best_choice $action
			set max_val $q_table_entry($index)
		}
	}
	return $best_choice
}


# -- for each entry in index_q_table add entry for behaviour != 0, assume that profit is in proportion to trick_rel
proc coala_init_q_table_entries_tricks {agent} {
	global q_table_index poss_behaviour q_table_entry q_visited 

	foreach situation $q_table_index($agent) {
		set index1 [concat $agent $situation 0]
		set entry_coop $q_table_entry($index1)
		foreach action $poss_behaviour($agent) {
			if {$action != 0} {
				set index2 [concat $agent $situation $action]
				set q_table_entry($index2) 0
				set q_visited($index2) 0
			}
		}
	}
}

# -- if encounter new situation, make guess for expectations

proc coala_init_q_table_entries {agent situation} {
	global q_table_index poss_behaviour q_table_entry q_visited 

	foreach action $poss_behaviour($agent) {
		set index [concat $agent $situation $action]
		set q_table_entry($index) 0
		set q_visited($index) 0
	}
	set q_table_index($agent) [lsort -integer [lappend q_table_index($agent) $situation]]
}

# check if action might not lead to too high trust loss 
# trust must not be negative for any other agent currently, and its sum not become negative as result of action
# an action other than 0 is allowed if 
# 1. the estimated trust loss does not lower the game situation below 0 and if 
# 2. the estimated trust of every agent for self is > 0
proc coala_opp_action_allowed {agent situation action} {
	global infos_in_nego trust_loss not_trusted
	global q_tables_file

	set index [concat $agent $action]
	if {$action == 0} {
		return 1
	} else {
		# -- avoid negative trust from anyone
		if $not_trusted($agent) {
			return 0
		} 
		if {![info exists trust_loss($index)]} {
			set trust_loss2 [coala_estimate_trust_loss  $agent [expr $action - 1] 1]
		} else {
			set trust_loss2 $trust_loss($index)
		}
		if {$situation - $trust_loss2 > 0} {
			return 1
		} else {
			return 0
		}
	}
}

proc coala_estimate_trust_loss {agent action i} {
	global  trust_loss

	set index [concat $agent $action]
	if {![info exists trust_loss($index)]} {
		incr i
		coala_estimate_trust_loss $agent [expr $action - 1] $i
	} else {
		return $trust_loss($index)
	}
}

