# -----------------------------------------------------------------------------
# Copyright (c) 2009 by Christine Walter, Matthias Klusch
# Saarland University and German Research Center for Artificial Intelligence,
# Saarbruecken, Germany
#
# All rights reserved. No warranties will be given on any issues arising out 
# of the use of this software product. Permission to use, copy, modify and 
# distribute this software product for non-commercial purposes is hereby 
# granted, provided that the above copyright notice appears in all copies and 
# respective publications.
# All commercial trades with this product without specific written prior 
# permission are prohibited.
# -----------------------------------------------------------------------------

proc coala_write_q_table_files {neg_no} {
	global agents_list is_opp poss_behaviour q_table_index q_table_entry q_visited neg_number
	global trust_loss actions_tried simulation_name total_opp_actions
	global coal_offers_received new_coals_made q_tables_file
	global total_opp_best_choice total_good_best_choice total_opp_actions total_good_actions
	global earliest_no_learning latest_learning

	foreach agent $agents_list {
		if {$is_opp($agent)} {
			set q_tables_file($agent) [open "logs/$simulation_name\.q_tables_$agent\.log" a]
			coala_puts $q_tables_file($agent) "q_table"
			coala_puts $q_tables_file($agent) "[llength $poss_behaviour($agent)] [llength $q_table_index($agent)] $neg_no"

			foreach situation $q_table_index($agent) {
				coala_puts $q_tables_file($agent) "$situation"
				set row {}
				foreach action $poss_behaviour($agent) {
					set index [concat $agent $situation $action]
					if [info exists q_table_entry($index)] {
						lappend row [list $action [format "%.3f" $q_table_entry($index)] $q_visited($index)]
					}
				}
				coala_puts $q_tables_file($agent) "$row"
			}
			coala_puts $q_tables_file($agent) "end_q_table"

			coala_puts $q_tables_file($agent) "trust_loss"
			foreach action $poss_behaviour($agent) {
				set index2 [concat $agent $action]
				if {[info exists trust_loss($index2)]} {
					set row [concat $action $trust_loss($agent $action) [lindex $actions_tried($agent) $action]]
					coala_puts $q_tables_file($agent) "$row"
				}
			}
			coala_puts $q_tables_file($agent) "end_trust_loss"
			coala_puts $q_tables_file($agent) "end_neg $neg_number"
			close $q_tables_file($agent)    
		}
	}
}


proc coala_write_opp_statfile {} {
	global total_not_trusted total_learning total_best_action latest_learning earliest_no_learning 
	global agents_list is_opp poss_behaviour simulation_name

	set opp_stat_file [open "logs/$simulation_name\.opp_stats\.log" a]
	foreach agent $agents_list {
		if $is_opp($agent) {
			set line {}
			lappend line $agent
			lappend line $total_learning($agent)
			lappend line $total_not_trusted($agent)
			lappend line $latest_learning($agent)
			lappend line $earliest_no_learning($agent)
			lappend line $poss_behaviour($agent)
			foreach action $poss_behaviour($agent) {
				lappend line $total_best_action([list $agent $action])
			}
			coala_puts $opp_stat_file $line
		}
	}
	close $opp_stat_file
}

# -- assume same outcome as most similar known situation and action
proc coala_get_closest_q_entry {agent situation action} {
	global q_table_entry

	set lower_val [expr $situation - 1]
	set higher_val [expr $situation + 1]
	while {$lower_val != -1 && $higher_val != 1} {
		set lower_entry [concat $agent $lower_val $action]
		set higher_entry [concat $agent $higher_val $action]
		if [info exists q_table_entry($lower_entry)] {
			if [info exists q_table_entry($higher_entry)] {
				return [expr ( $q_table_entry($lower_entry) + $q_table_entry($higher_entry) ) / 2.0]
				} else {
					return $q_table_entry($lower_entry)
			}
			} else {
				if [info exists q_table_entry($higher_entry)] {
					return $q_table_entry($higher_entry)
					} else {
						set lower_val [expr $lower_val - 1]
						set higher_val [expr $higher_val + 1]
				}
		}
	}
}




# -- if coal-offer (i.e. my_coal in prefList of otherSingleAgentCoal) lower threshhold
proc coala_opp_update_trust_threshholds {} {
	global PrefList agents_list is_opp coal_struct total_utility_change num_building_coalition_nego trust_threshhold

	foreach agent $agents_list {
		if {$is_opp($agent) && [lsearch $coal_struct $agent] != -1} {
			foreach agent2 $agents_list {
				if {$agent == $agent2 || [lsearch $coal_struct $agent2] == -1} {
					continue
					} else {
		if {$num_building_coalition_nego($agent2 $agent) != 0} {
		set trust_val [expr $total_utility_change($agent2 $agent) / $num_building_coalition_nego($agent2 $agent)]
		} else {
			set trust_val 0
		}
		set coal_offer 0
		foreach offer $PrefList($agent2) {
			if {[lindex $offer 0] == $agent} {
				set coal_offer 1
			}
		}
		if {$trust_val < $trust_threshhold($agent $agent2) && $coal_offer} {
			set trust_threshhold([list $agent $agent2]) $trust_val
		}
		if {$trust_val > $trust_threshhold($agent $agent2) && !$coal_offer && [coala_max_pot_gain $agent $agent2]} {
			set trust_threshhold([list $agent $agent2]) $trust_val
		}
				}
			}
		}
	}
}

# -- if both want all infos of other, there is maximal potential gain for a coalition
proc coala_max_pot_gain {agent agent2} {
	global infos_in_nego

	set i 1
	foreach i_unit $infos_in_nego($agent) {
		if {[lsearch [lindex $i_unit 2] $agent2] == -1} {
			set i 0
		}
	}
	foreach i_unit $infos_in_nego($agent2) {
		if {[lsearch [lindex $i_unit 2] $agent] == -1} {
			set i 0
		}
	}
	return $i
}






proc coala_open_q_files {} {
	global agents_list is_opp q_tables_file simulation_name

	foreach agent $agents_list {
		if {$is_opp($agent)} {
			set q_tables_file($agent) [open "logs/$simulation_name\.q_table_$agent\.log" w]
		}
	}
}

proc coala_close_q_files {} {
	global agents_list is_opp q_tables_file total_opp_actions

	foreach agent $agents_list {
		if {$is_opp($agent)} {
			close $q_tables_file($agent)
		}
	}
}



proc coala_get_agent_types_from_logfile {} {
	global num_agents

	# ===== opt_agent: 
	# ===== 0 - latecomer
	# ===== 1 - dropout
	# ===== 2 - rel begin
	# ===== 3 - irrel during
	# ===== 4 - rel during
	# ===== 5 - prob period
	# ===== 6 - act mal
	# ===== 7 - is opp
	# ===== 8 - trick rel

	global agents_list opt_agent act_mal is_opp

	foreach agent $agents_list {
		if [lindex 6 $opt_agent($agent)] {
			set act_mal($agent) 1
		} else {
			set act_mal($agent) 0
		}
		if [lindex 7 $opt_agent($agent)] {
			set is_opp($agent) 1
		} else {
			set is_opp($agent) 0
		}
	}


}