# -----------------------------------------------------------------------------
# Copyright (c) 2009 by Christine Walter, Matthias Klusch
# Saarland University and German Research Center for Artificial Intelligence,
# Saarbruecken, Germany
#
# All rights reserved. No warranties will be given on any issues arising out 
# of the use of this software product. Permission to use, copy, modify and 
# distribute this software product for non-commercial purposes is hereby 
# granted, provided that the above copyright notice appears in all copies and 
# respective publications.
# All commercial trades with this product without specific written prior 
# permission are prohibited.
# -----------------------------------------------------------------------------


# -- procedures related to BSCA-RC

proc coala_compute_reliability_of_coals {agents_in_nego coal_struct} {
	global nego_start_time act_mal logfile rely_coal is_opp

	set start_time [clock clicks]	
	set max_run_time 0
	foreach agent $agents_in_nego {
		set ag_start_time [clock clicks]
		set coal [coala_get_coal_of_agent $agent]
		set rely_list($agent) {}
		foreach other_coal $coal_struct {
			if {$other_coal == $coal} {
				continue
			}
			set index_c [list $agent $other_coal]
			# ===== malicious agent considers all other coalitions as trustworthy, opps are a bit more clever
			if {$act_mal($agent) && !$is_opp($agent)} {
				set rely_coal($index_c) 1
				lappend rely_list($agent) 1
				continue
			} else {
				set rely_sum 0
				foreach o_agent $other_coal {
					set index [list $agent $o_agent]
					set rely($index) [coala_get_reliability $agent $o_agent]
					set rely_sum [expr $rely_sum + $rely($index)]
				}
				set coal_size [llength $other_coal]
				set rely_coal($index_c) [expr $rely_sum / $coal_size]
				lappend rely_list($agent) $rely_coal($index_c)
			}
		}
		coala_puts $logfile "$agent $rely_list($agent)"
		set ag_end_time [clock clicks]
		if {[expr $ag_end_time - $ag_start_time] > $max_run_time} {
			set max_run_time [expr $ag_end_time - $ag_start_time]
		}
	}
	set end_time [clock clicks]
	set nego_start_time [expr $nego_start_time + ($end_time - $start_time - $max_run_time)] 

}

proc coala_get_reliability {agent agent2} {
	global trial_time num_building_coalition_nego uc_vals reputation 
	global total_utility_change num_building_coalition_nego
	global case_uc_agent case_uc_gs case_uc_ro case_uc total_cases this_neg_gs this_neg_ro
	global pred_uc

	set index [list $agent $agent2]
	# -- if trial time not over, give agent a chance and consider it reliable by default
	if {$trial_time($agent) >= $num_building_coalition_nego($index)} {
		set pred_uc($index) [list 1 "probation"]
		return 1.0
	}
	if {![info exists this_neg_gs($index)] || ![info exists this_neg_ro($index)]} {
		set pred_uc($index) [list 1 "nodata"]
		return 1.0
	}

	# -- from experiences so far, compute expected profit from agent2 using Naive Bayes
	set expected_uc 0
	set max_prob 0
	set gs [expr $this_neg_gs($index) + 0.0]
	set ro $this_neg_ro($index)
	# -- to avoid division by zero, return 1.0 if gs is 0 (since the expected profit is 0 anyway, this is considered very reliable information)
	if {$gs == 0} {
		set pred_uc($index) [list 1 "gszero"]
		return 1.0
	}

	foreach uc $uc_vals($agent) {
		set index_a [list $agent $agent2 $uc]
		set index_gs [list $agent $gs $uc]
		set index_ro [list $agent $ro $uc]
		set index_uc [list $agent $uc]
		if {![info exists case_uc_agent($index_a)] || ![info exists case_uc_gs($index_gs)] || ![info exists case_uc_ro($index_ro)]} {
			continue
		} else {
			set prob_uc [expr ($case_uc($index_uc) / $total_cases($agent)) * ($case_uc_gs($index_gs) / $case_uc($index_uc)) * ($case_uc_ro($index_ro) / $case_uc($index_uc)) * ($case_uc_agent($index_a) / $case_uc($index_uc))]
		}
		if {$prob_uc > $max_prob} {
			set max_prob $prob_uc
			set expected_uc $uc
		}
	}
	# -- if there are not enough cases in db, use trust+rep /2 instead
	if {$max_prob == 0} {
		set expected_uc [expr ($reputation($index) + ($total_utility_change($index) / $num_building_coalition_nego($index))) / 2.0]
		set pred_uc($index) [list [coala_round $expected_uc] "tr"]
	} else {
		set pred_uc($index) [list [coala_round $expected_uc] "nbc"]
	}
	return [expr $expected_uc / $gs]
}

proc coala_get_mutual_proposals_rc {coals PList} {
	global logfile 

	upvar $PList PrefList

	set accepted_proposals {}
	set checked {}
	foreach coal1 $coals {
		for {set i 0} {$i < [llength $PrefList($coal1)]} {incr i} {
			set proposal1 [lindex $PrefList($coal1) $i]
			set desc_coal [lindex $proposal1 0]
			set weighted_profit [lindex $proposal1 1]
			set profit [lindex $proposal1 2]
			set proposal_index [lindex $proposal1 3]
			# ==== look up proposals of desc_coal
			for {set j 0} {$j < [llength $PrefList($desc_coal)]} {incr j} {
				set proposal2 [lindex $PrefList($desc_coal) $j] 
				set proposal_index2 [lindex $proposal2 3]
				set sum_index [expr $proposal_index + $proposal_index2]
				if {[lindex $proposal2 0] == $coal1} {
					# ===== to take only one proposal (without this we have two same proposals) 
					# -- if desc_coal hasn't been checked already, break; i.e. mutual proposal is taken up the second time we come across it; then we append it to the list with the profit from the 'smaller' coalition, i.e. the one with better rep
					if {[lsearch $checked $desc_coal] > -1} {
						if {$coal1 < $desc_coal} {
							lappend accepted_proposals [list [lsort [concat $coal1 $desc_coal]] $profit $coal1 $desc_coal $i $sum_index] 
						} else {
							lappend accepted_proposals [list [lsort [concat $coal1 $desc_coal]] $profit $desc_coal $coal1 $j $sum_index] 
						}
					}
					break
				}
			}
		}
		lappend checked $coal1
	}
	
	# ===== compare command for sorting the proposal list
	proc rank_compare {j i} {
		# ==== compare sum of indices (mutual first choices come first, and so on)
		if {[lindex $i 5] < [lindex $j 5]} {
			return 1
		} else {
			if {[lindex $i 5] > [lindex $j 5]} {
				return -1
			} else {
				# ==== compare the first agent name
				if {[lindex [lindex $i 0] 0] > [lindex [lindex $j 0] 0]} {
					return -1
				} else {
					if {[lindex [lindex $i 0] 0] < [lindex [lindex $j 0] 0]} {
						return 1
					} else {
						if {[lindex $i 4] > [lindex $j 4]} {
							return -1
						}
						return 1
					}
				}
			}
		}
	}
	
	# ===== sort the proposal list	 
	set accepted_proposals [lsort -command rank_compare $accepted_proposals]
	
	coala_puts $logfile "sorted_proposal_list"
	foreach proposal $accepted_proposals {
		coala_puts $logfile $proposal
	}
	return $accepted_proposals
}




